@@ -2,7 +2,7 @@
  * @file export.c  OPML feed list import & export
  *
  * Copyright (C) 2004-2006 Nathan J. Conrad <t98502@users.sourceforge.net>
- * Copyright (C) 2004-2012 Lars Windolf <lars.lindner@gmail.com>
+ * Copyright (C) 2004-2022 Lars Windolf <lars.windolf@gmx.de>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,10 +19,11 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include <sys/stat.h>
-
 #include "export.h"
 
+#include <errno.h>
+#include <glib.h>
+#include <glib/gstdio.h>
 #include <libxml/tree.h>
 
 #include "auth.h"
@@ -32,9 +33,10 @@
 #include "favicon.h"
 #include "feedlist.h"
 #include "folder.h"
+#include "node.h"
 #include "xml.h"
 #include "ui/ui_common.h"
-#include "ui/feed_list_node.h"
+#include "ui/feed_list_view.h"
 
 struct exportData {
 	gboolean	trusted; /**< Include all the extra Liferea-specific tags */
@@ -50,20 +52,19 @@ export_append_node_tag (nodePtr node, gpointer userdata)
 	xmlNodePtr 	cur = ((struct exportData*)userdata)->cur;
 	gboolean	internal = ((struct exportData*)userdata)->trusted;
 	xmlNodePtr	childNode;
-	gchar		*tmp;
 
 	/* When exporting external OPML do not export every node type... */
 	if (!(internal || (NODE_TYPE (node)->capabilities & NODE_CAPABILITY_EXPORT)))
 		return;
-	
+
 	childNode = xmlNewChild (cur, NULL, BAD_CAST"outline", NULL);
 
 	/* 1. write generic node attributes */
 	xmlNewProp (childNode, BAD_CAST"title", BAD_CAST node_get_title(node));
 	xmlNewProp (childNode, BAD_CAST"text", BAD_CAST node_get_title(node)); /* The OPML spec requires "text" */
 	xmlNewProp (childNode, BAD_CAST"description", BAD_CAST node_get_title(node));
-	
-	if (node_type_to_str (node)) 
+
+	if (node_type_to_str (node))
 		xmlNewProp (childNode, BAD_CAST"type", BAD_CAST node_type_to_str (node));
 
 	/* Don't add the following tags if we are exporting to other applications */
@@ -90,32 +91,22 @@ export_append_node_tag (nodePtr node, gpointer userdata)
 
 		if (FALSE == node->sortReversed)
 			xmlNewProp (childNode, BAD_CAST"sortReversed", BAD_CAST"false");
-			
+
 		if (node->loadItemLink)
 			xmlNewProp (childNode, BAD_CAST"loadItemLink", BAD_CAST"true");
-
-		/* Do not export the default view mode setting to avoid making 
-		   it permanent. Do not use node_get_view_mode () here to ensure
-		   that the comparison works as node_get_view_mode () returns
-		   the effective mode! */
-		if (NODE_VIEW_MODE_DEFAULT != node->viewMode) {
-			tmp = g_strdup_printf ("%u", node_get_view_mode(node));
-			xmlNewProp (childNode, BAD_CAST"viewMode", BAD_CAST tmp);
-			g_free (tmp);
-		}
 	}
-	
+
 	/* 2. add node type specific stuff */
 	NODE_TYPE (node)->export (node, childNode, internal);
-	
+
 	/* 3. add children */
 	if (internal) {
-		if (feed_list_node_is_expanded (node->id))
+		if (feed_list_view_is_expanded (node->id))
 			xmlNewProp (childNode, BAD_CAST"expanded", BAD_CAST"true");
 		else
 			xmlNewProp (childNode, BAD_CAST"collapsed", BAD_CAST"true");
 	}
-	
+
 	if (IS_FOLDER (node))
 		export_node_children (node, childNode, internal);
 }
@@ -124,7 +115,7 @@ static void
 export_node_children (nodePtr node, xmlNodePtr cur, gboolean trusted)
 {
 	struct exportData	params;
-	
+
 	params.cur = cur;
 	params.trusted = trusted;
 	node_foreach_child_data (node, export_append_node_tag, &params);
@@ -140,49 +131,49 @@ export_OPML_feedlist (const gchar *filename, nodePtr node, gboolean trusted)
 	int		old_umask = 0;
 
 	debug_enter ("export_OPML_feedlist");
-	
+
 	backupFilename = g_strdup_printf ("%s~", filename);
-	
-	doc = xmlNewDoc ("1.0");
-	if (doc) {	
+
+	doc = xmlNewDoc (BAD_CAST"1.0");
+	if (doc) {
 		opmlNode = xmlNewDocNode (doc, NULL, BAD_CAST"opml", NULL);
 		if (opmlNode) {
 			xmlNewProp (opmlNode, BAD_CAST"version", BAD_CAST"1.0");
-			
+
 			/* create head */
 			cur = xmlNewChild (opmlNode, NULL, BAD_CAST"head", NULL);
 			if (cur)
 				xmlNewTextChild (cur, NULL, BAD_CAST"title", BAD_CAST"Liferea Feed List Export");
-			
+
 			/* create body with feed list */
 			cur = xmlNewChild (opmlNode, NULL, BAD_CAST"body", NULL);
 			if (cur)
 				export_node_children (node, cur, trusted);
-			
-			xmlDocSetRootElement (doc, opmlNode);		
+
+			xmlDocSetRootElement (doc, opmlNode);
 		} else {
 			g_warning ("could not create XML feed node for feed cache document!");
 			error = TRUE;
 		}
-		
+
 		if (!trusted)
 			old_umask = umask (022);	/* give read permissions for other, per-default we wouldn't give it... */
 
 		xmlSetDocCompressMode (doc, 0);
 
-		if (-1 == xmlSaveFormatFile (backupFilename, doc, TRUE)) {
+		if (-1 == xmlSaveFormatFileEnc (backupFilename, doc, "utf-8", TRUE)) {
 			g_warning ("Could not export to OPML file!");
 			error = TRUE;
 		}
-		
+
 		if (!trusted)
 			umask (old_umask);
-			
+
 		xmlFreeDoc (doc);
-		
+
 		if (!error) {
-			if (rename (backupFilename, filename) < 0) {
-				g_warning (_("Error renaming %s to %s\n"), backupFilename, filename);
+			if (g_rename (backupFilename, filename) < 0) {
+				g_warning (_("Error renaming %s to %s: %s\n"), backupFilename, filename, g_strerror (errno));
 				error = TRUE;
 			}
 		}
@@ -190,9 +181,9 @@ export_OPML_feedlist (const gchar *filename, nodePtr node, gboolean trusted)
 		g_warning ("Could not create XML document!");
 		error = TRUE;
 	}
-	
+
 	g_free (backupFilename);
-	
+
 	debug_exit ("export_OPML_feedlist");
 	return !error;
 }
@@ -205,25 +196,25 @@ import_parse_outline (xmlNodePtr cur, nodePtr parentNode, gboolean trusted)
 	nodePtr		node;
 	nodeTypePtr	type = NULL;
 	gboolean	needsUpdate = FALSE;
-	
+
 	debug_enter("import_parse_outline");
 
 	/* 1. determine node type */
-	typeStr = xmlGetProp (cur, BAD_CAST"type");
+	typeStr = (gchar *)xmlGetProp (cur, BAD_CAST"type");
 	if (typeStr) {
 		type = node_str_to_type (typeStr);
 		xmlFree (typeStr);
-	} 
-	
+	}
+
 	/* if we didn't find a type attribute we use heuristics */
 	if (!type) {
 		/* check for a source URL */
-		tmp = xmlGetProp (cur, BAD_CAST"xmlUrl");
+		tmp = (gchar *)xmlGetProp (cur, BAD_CAST"xmlUrl");
 		if (!tmp)
-			tmp = xmlGetProp (cur, BAD_CAST"xmlurl");	/* AmphetaDesk */
+			tmp = (gchar *)xmlGetProp (cur, BAD_CAST"xmlurl");	/* AmphetaDesk */
 		if (!tmp)
-			tmp = xmlGetProp (cur, BAD_CAST"xmlURL");	/* LiveJournal */
-		
+			tmp = (gchar *)xmlGetProp (cur, BAD_CAST"xmlURL");	/* LiveJournal */
+
 		if (tmp) {
 			debug0 (DEBUG_CACHE, "-> URL found assuming type feed");
 			type = feed_get_node_type();
@@ -234,23 +225,23 @@ import_parse_outline (xmlNodePtr cur, nodePtr parentNode, gboolean trusted)
 			debug0 (DEBUG_CACHE, "-> must be a folder");
 		}
 	}
-	
+
 	g_assert (NULL != type);
 
 	/* Check if adding this type is allowed */
 	// FIXME: Prevent news bins outside root source
 	// FIXME: Prevent search folders outside root source
-	
-	/* 2. do general node parsing */	
+
+	/* 2. do general node parsing */
 	node = node_new (type);
 	node_set_parent (node, parentNode, -1);
-	
+
 	/* The id should only be used from feedlist.opml. Otherwise,
 	   it could cause corruption if the same id was imported
 	   multiple times. */
 	if (trusted) {
 		gchar *id = NULL;
-		id = xmlGetProp (cur, BAD_CAST"id");
+		id = (gchar *)xmlGetProp (cur, BAD_CAST"id");
 
 		/* If, for some reason, the OPML has been corrupted
 		   and there are two copies asking for a certain ID
@@ -269,13 +260,13 @@ import_parse_outline (xmlNodePtr cur, nodePtr parentNode, gboolean trusted)
 	} else {
 		needsUpdate = TRUE;
 	}
-	
+
 	/* title */
-	title = xmlGetProp (cur, BAD_CAST"title");
-	if (!title || !xmlStrcmp (title, BAD_CAST"")) {
+	title = (gchar *)xmlGetProp (cur, BAD_CAST"title");
+	if (!title || !xmlStrcmp ((xmlChar *)title, BAD_CAST"")) {
 		if (title)
 			xmlFree (title);
-		title = xmlGetProp (cur, BAD_CAST"text");
+		title = (gchar *)xmlGetProp (cur, BAD_CAST"text");
 	}
 
 	if (title) {
@@ -284,51 +275,44 @@ import_parse_outline (xmlNodePtr cur, nodePtr parentNode, gboolean trusted)
 	}
 
 	/* sorting order */
-	sortStr = xmlGetProp (cur, BAD_CAST"sortColumn");
+	sortStr = (gchar *)xmlGetProp (cur, BAD_CAST"sortColumn");
 	if (sortStr) {
-		if (!xmlStrcmp (sortStr, "title"))
+		if (!xmlStrcmp ((xmlChar *)sortStr, BAD_CAST"title"))
 			node->sortColumn = NODE_VIEW_SORT_BY_TITLE;
-		else if (!xmlStrcmp (sortStr, "parent"))
+		else if (!xmlStrcmp ((xmlChar *)sortStr, BAD_CAST"parent"))
 			node->sortColumn = NODE_VIEW_SORT_BY_PARENT;
-		else if (!xmlStrcmp (sortStr, "state"))
+		else if (!xmlStrcmp ((xmlChar *)sortStr, BAD_CAST"state"))
 			node->sortColumn = NODE_VIEW_SORT_BY_STATE;
 		else
 			node->sortColumn = NODE_VIEW_SORT_BY_TIME;
 		xmlFree (sortStr);
 	}
-	sortStr = xmlGetProp (cur, BAD_CAST"sortReversed");
+	sortStr = (gchar *)xmlGetProp (cur, BAD_CAST"sortReversed");
 	if (sortStr) {
-		if(!xmlStrcmp (sortStr, BAD_CAST"false"))
+		if(!xmlStrcmp ((xmlChar *)sortStr, BAD_CAST"false"))
 			node->sortReversed = FALSE;
 		xmlFree (sortStr);
 	}
-	
+
 	/* auto item link loading flag */
-	tmp = xmlGetProp (cur, BAD_CAST"loadItemLink");
+	tmp = (gchar *)xmlGetProp (cur, BAD_CAST"loadItemLink");
 	if (tmp) {
-		if (!xmlStrcmp (tmp, BAD_CAST"true"))
+		if (!xmlStrcmp ((xmlChar *)tmp, BAD_CAST"true"))
 		node->loadItemLink = TRUE;
 		xmlFree (tmp);
 	}
 
-	/* viewing mode */
-	tmp = xmlGetProp (cur, BAD_CAST"viewMode");
-	if (tmp) {
-		node_set_view_mode (node, atoi (tmp));
-		xmlFree (tmp);
-	}
-	
-	/* expansion state */		
+	/* expansion state */
 	if (xmlHasProp (cur, BAD_CAST"expanded"))
 		node->expanded = TRUE;
 	else if (xmlHasProp (cur, BAD_CAST"collapsed"))
 		node->expanded = FALSE;
-	else 
+	else
 		node->expanded = TRUE;
-	
+
 	/* 3. Try to load the favicon (needs to be done before adding to the feed list) */
-	node_set_icon (node, favicon_load_from_cache (node->id));
-			
+	node_load_icon (node);
+
 	/* 4. add to GUI parent */
 	feedlist_node_imported (node);
 
@@ -338,28 +322,21 @@ import_parse_outline (xmlNodePtr cur, nodePtr parentNode, gboolean trusted)
 		while (child) {
 			if (!xmlStrcmp (child->name, BAD_CAST"outline"))
 				import_parse_outline (child, node, trusted);
-			child = child->next;				
+			child = child->next;
 		}
 	}
-	
+
 	/* 6. do node type specific parsing */
 	NODE_TYPE (node)->import (node, parentNode, cur, trusted);
 
-	if (node->subscription) {
-		/* Handle OPML auth info (imported from subscription_import() */
-		if(node->subscription->updateOptions->username) {
-			/* Write to password store (for migration) */
-			liferea_auth_info_store (node->subscription);
-		} else {
-			/* If no auth options in OPML try to import them from the key store */
-			liferea_auth_info_query (node->id);
-		}
+	if (node->subscription)
+		liferea_auth_info_query (node->id);
 
-		/* 7. update immediately if necessary */
-		if (needsUpdate) {
-			debug1 (DEBUG_CACHE, "seems to be an import, setting new id: %s and doing first download...", node_get_id(node));
-			subscription_update (node->subscription, 0);
-		}
+	/* 7. update immediately if necessary */
+	// FIXME: this should not be done here!!!
+	if (node->subscription && needsUpdate) {
+		debug1 (DEBUG_CACHE, "seems to be an import, setting new id: %s and doing first download...", node_get_id(node));
+		subscription_update (node->subscription, 0);
 	}
 
 	/* 8. Always update the node info in the DB to ensure a proper
@@ -374,7 +351,7 @@ static void
 import_parse_body (xmlNodePtr n, nodePtr parentNode, gboolean trusted)
 {
 	xmlNodePtr cur;
-	
+
 	cur = n->xmlChildrenNode;
 	while (cur) {
 		if (!xmlStrcmp (cur->name, BAD_CAST"outline"))
@@ -387,7 +364,7 @@ static void
 import_parse_OPML (xmlNodePtr n, nodePtr parentNode, gboolean trusted)
 {
 	xmlNodePtr cur;
-	
+
 	cur = n->xmlChildrenNode;
 	while (cur) {
 		/* we ignore the head */
@@ -395,7 +372,7 @@ import_parse_OPML (xmlNodePtr n, nodePtr parentNode, gboolean trusted)
 			import_parse_body (cur, parentNode, trusted);
 		}
 		cur = cur->next;
-	}	
+	}
 }
 
 gboolean
@@ -404,9 +381,9 @@ import_OPML_feedlist (const gchar *filename, nodePtr parentNode, gboolean showEr
 	xmlDocPtr 	doc;
 	xmlNodePtr 	cur;
 	gboolean	error = FALSE;
-	
+
 	debug1 (DEBUG_CACHE, "Importing OPML file: %s", filename);
-	
+
 	/* read the feed list */
 	doc = xmlParseFile (filename);
 	if (!doc) {
@@ -426,16 +403,16 @@ import_OPML_feedlist (const gchar *filename, nodePtr parentNode, gboolean showEr
 		} else {
 			if (!trusted) {
 				/* set title only when importing as folder and not as OPML source */
-				xmlNodePtr title = xpath_find (cur, "/opml/head/title"); 
+				xmlNodePtr title = xpath_find (cur, "/opml/head/title");
 				if (title) {
 					xmlChar *titleStr = xmlNodeListGetString (title->doc, title->xmlChildrenNode, 1);
 					if (titleStr) {
-						node_set_title (parentNode, titleStr);
+						node_set_title (parentNode, (gchar *)titleStr);
 						xmlFree (titleStr);
 					}
 				}
 			}
-		
+
 			while (cur) {
 				if (!xmlIsBlankNode (cur)) {
 					if (!xmlStrcmp (cur->name, BAD_CAST"opml")) {
@@ -452,7 +429,7 @@ import_OPML_feedlist (const gchar *filename, nodePtr parentNode, gboolean showEr
 		}
 		xmlFreeDoc (doc);
 	}
-	
+
 	return !error;
 }
 
@@ -460,12 +437,12 @@ import_OPML_feedlist (const gchar *filename, nodePtr parentNode, gboolean showEr
 
 static void
 on_import_activate_cb (const gchar *filename, gpointer user_data)
-{	
+{
 	if (filename) {
 		nodePtr node = node_new (folder_get_node_type ());
 		node_set_title (node, _("Imported feed list"));
 		feedlist_node_added (node);
-		
+
 		if (!import_OPML_feedlist (filename, node, TRUE /* show errors */, FALSE /* not trusted */)) {
 			feedlist_remove_node (node);
 		}
@@ -484,7 +461,7 @@ on_export_activate_cb (const gchar *filename, gpointer user_data)
 	if (filename) {
 		if (!export_OPML_feedlist (filename, feedlist_get_root (), FALSE))
 			ui_show_error_box (_("Error while exporting feed list!"));
-		else 
+		else
 			ui_show_info_box (_("Feed List exported!"));
 	}
 }
