@@ -1,8 +1,8 @@
-/**
- * @file itemlist.c  itemlist handling
+/*
+ * @file itemlist.c  item list handling
+ *
+ * Copyright (C) 2004-2023 Lars Windolf <lars.windolf@gmx.de>
  *
- * Copyright (C) 2004-2012 Lars Lindner <lars.lindner@gmail.com>
- *	      
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -29,6 +29,7 @@
 #include "feed.h"
 #include "feedlist.h"
 #include "folder.h"
+#include "item_history.h"
 #include "item_state.h"
 #include "itemlist.h"
 #include "itemset.h"
@@ -40,12 +41,10 @@
 #include "ui/itemview.h"
 #include "ui/liferea_shell.h"
 #include "ui/feed_list_view.h"
-#include "ui/liferea_htmlview.h"
-#include "ui/ui_node.h"
 
-/* The 'item list' is a controller for 'item view' and database backend. 
+/* The 'item list' is a controller for 'item view' and database backend.
    It manages the currently displayed 'node', realizes filtering
-   by node and 'item set' rules, also duplicate elimination and provides 
+   by node and 'item set' rules, also duplicate elimination and provides
    synchronisation for backend and GUI access to the current itemset.
 
    The 'item list' provides methods to add/remove items from the 'item
@@ -56,27 +55,32 @@
    is done by the 'item view'.
  */
 
-#define ITEMLIST_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), ITEMLIST_TYPE, ItemListPrivate))
+#define ITEMLIST_GET_PRIVATE itemlist_get_instance_private
 
 struct ItemListPrivate
 {
-	GHashTable	*guids;			/**< list of GUID to avoid having duplicates in currently loaded list */
-	itemSetPtr	filter;			/**< currently active filter rules */
-	nodePtr		currentNode;		/**< the node whose own or its child items are currently displayed */
-	gulong		selectedId;		/**< the currently selected (and displayed) item id */
-	
-	nodeViewType	viewMode;		/**< current viewing mode */
-	guint 		loading;		/**< if >0 prevents selection effects when loading the item list */
-	itemPtr		invalidSelection;	/**< if set then the next selection might need to do an unselect first */
+	GHashTable	*guids;			/*<< list of GUID to avoid having duplicates in currently loaded list */
+	itemSetPtr	filter;			/*<< currently active filter rules */
+	nodePtr		currentNode;		/*<< the node whose own or its child items are currently displayed */
+	gulong		selectedId;		/*<< the currently selected (and displayed) item id */
 
-	gboolean 	deferredRemove;		/**< TRUE if selected item needs to be removed from cache on unselecting */
-	gboolean 	deferredFilter;		/**< TRUE if selected item needs to be filtered on unselecting */
+	guint 		loading;		/*<< if >0 prevents selection effects when loading the item list */
+	itemPtr		invalidSelection;	/*<< if set then the next selection might need to do an unselect first */
+
+	gboolean 	deferredRemove;		/*<< TRUE if selected item needs to be removed from cache on unselecting */
 };
 
+enum {
+	ITEM_UPDATED,	/*<< state of a currently visible item has changed */
+	LAST_SIGNAL
+};
+
+static guint itemlist_signals[LAST_SIGNAL] = { 0 };
+
 static GObjectClass *parent_class = NULL;
 static ItemList *itemlist = NULL;
 
-G_DEFINE_TYPE (ItemList, itemlist, G_TYPE_OBJECT);
+G_DEFINE_TYPE_WITH_CODE (ItemList, itemlist, G_TYPE_OBJECT, G_ADD_PRIVATE (ItemList));
 
 static void
 itemlist_init (ItemList *il)
@@ -143,7 +147,17 @@ itemlist_class_init (ItemListClass *klass)
 
 	object_class->finalize = itemlist_finalize;
 
-	g_type_class_add_private (object_class, sizeof(ItemListPrivate));
+	itemlist_signals[ITEM_UPDATED] =
+		g_signal_new ("item-updated",
+		G_OBJECT_CLASS_TYPE (object_class),
+		(GSignalFlags)(G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION),
+		0,
+		NULL,
+		NULL,
+		g_cclosure_marshal_VOID__STRING,
+		G_TYPE_NONE,
+		1,
+		G_TYPE_STRING);
 }
 
 /* member wrappers */
@@ -169,67 +183,84 @@ itemlist_set_selected (itemPtr item)
 nodePtr
 itemlist_get_displayed_node (void)
 {
-	return itemlist->priv->currentNode; 
+	return itemlist->priv->currentNode;
 }
 
-/* called when unselecting the item or unloading the item list */
-static void
-itemlist_check_for_deferred_action (void) 
-{
-	itemPtr	item;
-	
-	if(itemlist->priv->selectedId) {
-		gulong id = itemlist->priv->selectedId;
-		itemlist_set_selected(NULL);
-
-		/* check for removals caused by itemlist filter rule */
-		if(itemlist->priv->deferredFilter) {
-			itemlist->priv->deferredFilter = FALSE;
-			item = item_load(id);
-			itemview_remove_item(item);
-			ui_node_update(item->nodeId);
-		}
-
-		/* check for removals caused by vfolder rules */
-		if(itemlist->priv->deferredRemove) {
-			itemlist->priv->deferredRemove = FALSE;
-			item = item_load(id);
-			itemlist_remove_item(item);
-		}
-	}
-}
-
-// FIXME: is this an item set method?
 static gboolean
 itemlist_filter_check_item (itemPtr item)
 {
-	/* use search folder rule list in case of a search folder */
 	if (itemlist->priv->currentNode && IS_VFOLDER (itemlist->priv->currentNode)) {
 		vfolderPtr vfolder = (vfolderPtr)itemlist->priv->currentNode->data;
+
+		/* Use hide read for search folders */
+		if (vfolder->unreadOnly && item->readStatus)
+			return FALSE;
+
+		/* Use search folder rule list in case of a search folder */
 		return itemset_check_item (vfolder->itemset, item);
 	}
 
 	/* apply the item list filter if available */
 	if (itemlist->priv->filter)
 		return itemset_check_item (itemlist->priv->filter, item);
-	
+
 	/* otherwise keep the item */
 	return TRUE;
 }
 
+/* called when unselecting the item or unloading the item list */
 static void
-itemlist_merge_item (itemPtr item) 
+itemlist_check_for_deferred_action (void)
+{
+	itemPtr item;
+
+	if (!itemlist_get_selected_id ())
+		return;
+
+	item = itemlist_get_selected ();
+	itemlist_set_selected (NULL);
+
+	/* check for item hiding caused by itemlist filter rule (i.e. folder hide read items) */
+	if (!itemlist_filter_check_item (item)) {
+		gboolean keep_for_search_folder;
+		conf_get_bool_value (DEFER_DELETE_MODE, &keep_for_search_folder);
+		if (keep_for_search_folder) {
+			item->isHidden = TRUE;
+			itemview_update_item (item);
+		} else {
+			itemview_remove_item (item);
+		}
+		feed_list_view_update_node (item->nodeId);
+	}
+
+	/* check for item unloading caused by search folder rules (i.e. unread items only) */
+	if (itemlist->priv->deferredRemove) {
+		itemlist->priv->deferredRemove = FALSE;
+		itemlist_remove_item (item);
+	} else {
+		item_unload (item);
+	}
+}
+
+static void
+itemlist_merge_item (itemPtr item)
 {
 	if (!itemlist_duplicate_list_check_item (item))
-		return;		
-		
+		return;
+
 	if (!itemlist_filter_check_item (item))
 		return;
-		
+
 	itemlist_duplicate_list_add_item (item);
 	itemview_add_item (item);
 }
 
+static void
+itemlist_merge_item_callback (itemPtr item, gpointer _unused)
+{
+	itemlist_merge_item (item);
+}
+
 /* Helper method checking if the passed item set is relevant
    for the currently item list content. */
 static gboolean
@@ -242,33 +273,29 @@ itemlist_itemset_is_valid (itemSetPtr itemSet)
 
 	if (!itemlist->priv->currentNode)
 		return FALSE; /* Nothing to do if nothing is displayed */
-		
+
 	if (!IS_VFOLDER (itemlist->priv->currentNode) &&
-	    (itemlist->priv->currentNode != node) && 
+	    (itemlist->priv->currentNode != node) &&
 	    !node_is_ancestor (itemlist->priv->currentNode, node))
 		return FALSE; /* Nothing to do if the item set does not belong to this node, or this is a search folder */
 
 	conf_get_int_value (FOLDER_DISPLAY_MODE, &folder_display_mode);
 	if (IS_FOLDER (itemlist->priv->currentNode) && !folder_display_mode)
 		return FALSE; /* Bail out if it is a folder without the recursive display preference set */
-		
+
 	debug1 (DEBUG_GUI, "reloading item list with node \"%s\"", node_get_title (node));
 
 	return TRUE;
 }
 
-/**
- * To be called whenever an itemset was updated. If it is the
- * displayed itemset it will be merged against the item view.
- */
 void
-itemlist_merge_itemset (itemSetPtr itemSet) 
+itemlist_merge_itemset (itemSetPtr itemSet)
 {
 	debug_enter ("itemlist_merge_itemset");
-	
+
 	if (itemlist_itemset_is_valid (itemSet)) {
 		debug_start_measurement (DEBUG_GUI);
-		itemset_foreach (itemSet, itemlist_merge_item);
+		itemset_foreach (itemSet, itemlist_merge_item_callback, NULL);
 		itemview_update ();
 		debug_end_measurement (DEBUG_GUI, "itemlist merge");
 	}
@@ -276,61 +303,53 @@ itemlist_merge_itemset (itemSetPtr itemSet)
 	debug_exit ("itemlist_merge_itemset");
 }
 
-/** 
- * To be called whenever a node was selected and should
- * replace the current itemlist.
- */
 void
-itemlist_load (nodePtr node) 
+itemlist_load (nodePtr node)
 {
 	itemSetPtr	itemSet;
 	gint		folder_display_mode;
-	gboolean	folder_display_hide_read;
+	gboolean	display_hide_read = FALSE;
 
 	debug_enter ("itemlist_load");
 
 	g_return_if_fail (NULL != node);
-	
+
 	debug1 (DEBUG_GUI, "loading item list with node \"%s\"", node_get_title (node));
 
 	g_assert (!itemlist->priv->guids);
 	g_assert (!itemlist->priv->filter);
 
-	/* 1. Filter check. Don't continue if folder is selected and 
+	/* 1. Filter check. Don't continue if folder is selected and
 	   no folder viewing is configured. If folder viewing is enabled
 	   set up a "unread items only" rule depending on the prefences. */
 
-	/* for folders and other heirarchic nodes do filtering */
+	/* for folders and other hierarchic nodes do preference based filtering */
 	if (IS_FOLDER (node) || node->children) {
-		liferea_shell_update_allitems_actions (FALSE, 0 != node->unreadCount);
-
 		conf_get_int_value (FOLDER_DISPLAY_MODE, &folder_display_mode);
 		if (!folder_display_mode)
 			return;
-	
-		conf_get_bool_value (FOLDER_DISPLAY_HIDE_READ, &folder_display_hide_read);
-		if (folder_display_hide_read) {
-			itemlist->priv->filter = g_new0(struct itemSet, 1);
-			itemlist->priv->filter->anyMatch = TRUE;
-			itemset_add_rule (itemlist->priv->filter, "unread", "", TRUE);
-		}
-	} else {
-		liferea_shell_update_allitems_actions (0 != node->itemCount, 0 != node->unreadCount);
+
+		conf_get_bool_value (FOLDER_DISPLAY_HIDE_READ, &display_hide_read);
+	}
+
+	/* for search folders do properties based filtering */
+	if (IS_VFOLDER (node))
+		display_hide_read = ((vfolderPtr)node)->unreadOnly;
+
+	if (display_hide_read) {
+		itemlist->priv->filter = g_new0(struct itemSet, 1);
+		itemlist->priv->filter->anyMatch = TRUE;
+		itemset_add_rule (itemlist->priv->filter, "unread", "", TRUE);
 	}
 
 	itemlist->priv->loading++;
-	itemlist->priv->viewMode = node_get_view_mode (node);
-	itemview_set_layout (itemlist->priv->viewMode);
 
 	/* Set the new displayed node... */
 	itemlist->priv->currentNode = node;
 	itemview_set_displayed_node (itemlist->priv->currentNode);
 
-	if (NODE_VIEW_MODE_COMBINED != node_get_view_mode (node))
-		itemview_set_mode (ITEMVIEW_NODE_INFO);
-	else
-		itemview_set_mode (ITEMVIEW_ALL_ITEMS);
-	
+	itemview_set_mode (ITEMVIEW_NODE_INFO);
+
 	itemSet = node_get_itemset (itemlist->priv->currentNode);
 	itemlist_merge_itemset (itemSet);
 	if (!IS_VFOLDER (node))			/* FIXME: this is ugly! */
@@ -342,50 +361,35 @@ itemlist_load (nodePtr node)
 }
 
 void
-itemlist_unload (gboolean markRead) 
+itemlist_unload (void)
 {
 	/* Always clear to ensure clearing on search */
 	itemview_clear ();
 
 	if (itemlist->priv->currentNode) {
 		itemview_set_displayed_node (NULL);
-		
-		/* 1. Postprocessing for previously selected node, this is necessary
-		   to realize reliable read marking when using condensed mode. It's
-		   important to do this only when the selection really changed. */
-		if (markRead && (2 == node_get_view_mode (itemlist->priv->currentNode)))
-			feedlist_mark_all_read (itemlist->priv->currentNode);
-
 		itemlist_check_for_deferred_action ();
 	}
 
 	itemlist_set_selected (NULL);
 	itemlist_duplicate_list_free ();
 	itemlist->priv->currentNode = NULL;
-	
+
 	itemset_free (itemlist->priv->filter);
 	itemlist->priv->filter = NULL;
 }
 
 void
-itemlist_select_next_unread (void) 
+itemlist_select_next_unread (void)
 {
 	itemPtr	result = NULL;
 
-	/* If we are in combined mode we have to mark everything
-	   read or else we would never jump to the next feed,
-	   because no item will be selected and marked read... */
-	if (itemlist->priv->currentNode) {
-		if (NODE_VIEW_MODE_COMBINED == node_get_view_mode (itemlist->priv->currentNode))
-			node_mark_all_read (itemlist->priv->currentNode);
-	}
-
 	itemlist->priv->loading++;	/* prevent unwanted selections */
 
-	/* before scanning the feed list, we test if there is a unread 
+	/* before scanning the feed list, we test if there is a unread
 	   item in the currently selected feed! */
 	result = itemview_find_unread_item (itemlist->priv->selectedId);
-	
+
 	/* If none is found we continue searching in the feed list */
 	if (!result) {
 		nodePtr	node;
@@ -395,9 +399,7 @@ itemlist_select_next_unread (void)
 		if (node) {
 			/* load found feed */
 			feed_list_view_select (node);
-
-			if (NODE_VIEW_MODE_COMBINED != node_get_view_mode (node))
-				result = itemview_find_unread_item (0);	/* find first unread item */
+			result = itemview_find_unread_item (0);	/* find first unread item */
 		} else {
 			/* if we don't find a feed with unread items do nothing */
 			liferea_shell_set_status_bar (_("There are no unread items"));
@@ -405,7 +407,7 @@ itemlist_select_next_unread (void)
 	}
 
 	itemlist->priv->loading--;
-	
+
 	if (result)
 		itemview_select_item (result);
 }
@@ -413,17 +415,16 @@ itemlist_select_next_unread (void)
 /* menu commands */
 
 void
-itemlist_toggle_flag (itemPtr item) 
+itemlist_toggle_flag (itemPtr item)
 {
 	item_set_flag_state (item, !(item->flagStatus));
-	itemview_update ();
 }
 
 void
-itemlist_toggle_read_status (itemPtr item) 
+itemlist_toggle_read_status (itemPtr item)
 {
 	item_set_read_state (item, !(item->readStatus));
-	itemview_update ();
+	g_signal_emit_by_name (itemlist, "item-updated", item->nodeId);
 }
 
 /* function to remove items due to item list filtering */
@@ -434,36 +435,34 @@ itemlist_hide_item (itemPtr item)
 	   don't do it and set a flag to do it when unselecting */
 	if (itemlist->priv->selectedId != item->id) {
 		itemview_remove_item (item);
-		ui_node_update (item->nodeId);
+		feed_list_view_update_node (item->nodeId);
 	} else {
-		itemlist->priv->deferredFilter = TRUE;
 		/* update the item to show new state that forces
 		   later removal */
 		itemview_update_item (item);
 	}
 }
 
-/* function to cancel deferred removal of selected item */
+/* function to cancel deferred filtering of selected item */
 static void
 itemlist_unhide_item (itemPtr item)
 {
-	itemlist->priv->deferredFilter = FALSE;
+        item->isHidden = FALSE;
 }
 
 /* functions to remove items on remove requests */
 
 /* hard unconditional item remove */
 void
-itemlist_remove_item (itemPtr item) 
+itemlist_remove_item (itemPtr item)
 {
 	if (itemlist->priv->selectedId == item->id) {
 		itemlist_set_selected (NULL);
-		itemlist->priv->deferredFilter = FALSE;
 		itemlist->priv->deferredRemove = FALSE;
 	}
 
 	itemlist_duplicate_list_remove_item (item);
-		
+
 	itemview_remove_item (item);
 	itemview_update ();
 
@@ -472,14 +471,15 @@ itemlist_remove_item (itemPtr item)
 	/* update feed list counters*/
 	vfolder_foreach (node_update_counters);
 	node_update_counters (node_from_id (item->nodeId));
-	
+
 	item_unload (item);
+	g_signal_emit_by_name (itemlist, "item-updated", item->nodeId);
 }
 
 /* soft possibly delayed item remove */
-void
-itemlist_request_remove_item (itemPtr item) 
-{	
+static void
+itemlist_request_remove_item (itemPtr item)
+{
 	/* if the currently selected item should be removed we
 	   don't do it and set a flag to do it when unselecting */
 	if (itemlist->priv->selectedId != item->id) {
@@ -496,38 +496,34 @@ void
 itemlist_remove_items (itemSetPtr itemSet, GList *items)
 {
 	GList		*iter = items;
-	
+
 	while (iter) {
 		itemPtr item = (itemPtr) iter->data;
-
 		if (itemlist->priv->selectedId != item->id) {
-			/* don't call itemlist_remove_item() here, because it's to slow */
-			itemview_remove_item (item);
-			db_item_remove (item->id);
+			itemview_remove_item(item);
+			db_item_remove(item->id);
 		} else {
-			/* go the normal and selection-safe way to avoid disturbing the user */
-			itemlist_request_remove_item (item);
+			itemlist_request_remove_item(item);
 		}
-		item_unload (item);
+		g_object_unref (item);
+
 		iter = g_list_next (iter);
 	}
 
 	itemview_update ();
 	vfolder_foreach (node_update_counters);
 	node_update_counters (node_from_id (itemSet->nodeId));
+	g_signal_emit_by_name (itemlist, "item-updated", itemSet->nodeId);
 }
 
 void
 itemlist_remove_all_items (nodePtr node)
-{	
-	GList		*iter;
-	itemSetPtr	itemset;
-	
+{
 	if (node == itemlist->priv->currentNode)
 		itemview_clear ();
-		
+
 	db_itemset_remove_all (node->id);
-	
+
 	if (node == itemlist->priv->currentNode) {
 		itemview_update ();
 		itemlist_duplicate_list_free ();
@@ -535,6 +531,7 @@ itemlist_remove_all_items (nodePtr node)
 
 	vfolder_foreach (node_update_counters);
 	node_update_counters (node);
+	g_signal_emit_by_name (itemlist, "item-updated", node->id);
 }
 
 void
@@ -547,19 +544,11 @@ itemlist_update_item (itemPtr item)
 		itemlist_unhide_item (item);
 	}
 
-	/* FIXME: this is tricky. It's possible that the item
-	 * selected, but the itemview contains a webpage. In
-	 * that case, we don't want to reload the item, but if
-	 * the itemview contains the item, we want, as the
-	 * background will turn yellow and the the title will
-	 * turn red. So how to know whether the itemview contains
-	 * the item?
-	 */
 	itemview_update_item (item);
 }
 
 /* mouse/keyboard interaction callbacks */
-void 
+void
 itemlist_selection_changed (itemPtr item)
 {
 	debug_enter ("itemlist_selection_changed");
@@ -570,15 +559,15 @@ itemlist_selection_changed (itemPtr item)
 		   more matching the display rules because they have changed state */
 		itemlist_check_for_deferred_action ();
 
-		debug1 (DEBUG_GUI, "item list selection changed to \"%s\"", item_get_title (item));
-		
+		debug1 (DEBUG_GUI, "item list selection changed to \"%s\"", item?item_get_title (item):"(null)");
+
 		itemlist_set_selected (item);
-	
+
 		/* set read and unset update status when selecting */
 		if (item) {
 			gchar	*link = NULL;
 			nodePtr	node = node_from_id (item->nodeId);
-			
+
 			item_set_read_state (item, TRUE);
 			itemview_set_mode (ITEMVIEW_SINGLE_ITEM);
 
@@ -592,61 +581,54 @@ itemlist_selection_changed (itemPtr item)
 				itemview_select_item (item);
 				itemview_update ();
 			}
-			ui_node_update (item->nodeId);
+			feed_list_view_update_node (item->nodeId);
 		}
 
 		feedlist_reset_new_item_count ();
 	}
 
 	if (item)
-		item_unload (item);
-	
+		g_object_unref (item);
+
 	debug_end_measurement (DEBUG_GUI, "itemlist selection");
 	debug_exit ("itemlist_selection_changed");
 }
 
-/* viewing mode callbacks */
-
-guint
-itemlist_get_view_mode (void)
+static void
+itemlist_select_from_history (gboolean back)
 {
-	return itemlist->priv->viewMode; 
+	itemPtr item;
+	nodePtr node;
+
+	if (back)
+		item = item_history_get_previous ();
+	else
+		item = item_history_get_next ();
+
+	if (!item)
+		return;
+
+	node = node_from_id (item->parentNodeId);
+	if (!node)
+		return;
+
+	if (node != feedlist_get_selected ())
+		feed_list_view_select (node);
+
+	itemview_select_item (item);
+	item_unload (item);
 }
 
 void
-itemlist_set_view_mode (guint newMode)
+on_prev_read_item_activate (GSimpleAction *action, GVariant *parameter, gpointer user_data)
 {
-	nodePtr		node;
-	itemPtr		item;
-
-	itemlist->priv->viewMode = newMode;
-
-	node = itemlist_get_displayed_node ();
-	item = itemlist_get_selected ();
-
-	if (node) {
-		itemlist_unload (FALSE);
-
-		node_set_view_mode (node, itemlist->priv->viewMode);
-		itemview_set_layout (itemlist->priv->viewMode);
-		itemlist_load (node);
-
-		/* If there was an item selected, select it again since
-		 * itemlist_unload() unselects it.
-		 */
-		if (item && itemlist->priv->viewMode != NODE_VIEW_MODE_COMBINED)
-			itemview_select_item (item);
-	}
-
-	if (item)
-		item_unload (item);
+	itemlist_select_from_history (TRUE);
 }
 
 void
-on_view_activate (GtkRadioAction *action, GtkRadioAction *current, gpointer user_data)
+on_next_read_item_activate (GSimpleAction *action, GVariant *parameter, gpointer user_data)
 {
-	gint val = gtk_radio_action_get_current_value (current);
-	itemlist_set_view_mode (val);
+	itemlist_select_from_history (FALSE);
 }
 
 /* item loader methods */
@@ -675,7 +657,7 @@ itemlist_item_batch_fetched_cb (ItemLoader *il, GSList *items, gpointer user_dat
 	g_slist_free (items);
 }
 
-void
+static void
 itemlist_add_loader (ItemLoader *loader)
 {
 	g_signal_connect (G_OBJECT (loader), "item-batch-fetched", G_CALLBACK (itemlist_item_batch_fetched_cb), NULL);
@@ -686,16 +668,7 @@ itemlist_add_loader (ItemLoader *loader)
 void
 itemlist_add_search_result (ItemLoader *loader)
 {
-	nodeViewType viewMode;
-
-	/* Ensure that we are in a useful viewing mode (3 paned) */
-	itemlist_unload (FALSE);
-
-	viewMode = itemlist_get_view_mode ();
-	if ((NODE_VIEW_MODE_NORMAL != viewMode) &&
-	    (NODE_VIEW_MODE_WIDE != viewMode))
-		itemview_set_layout (NODE_VIEW_MODE_NORMAL);
-
+	itemlist_unload ();
 	itemview_set_mode (ITEMVIEW_SINGLE_ITEM);
 
 	/* Set current node to search result dummy node so that
@@ -711,4 +684,3 @@ itemlist_create (void)
 {
 	return ITEMLIST (g_object_new (ITEMLIST_TYPE, NULL));
 }
-
